@@ -16,6 +16,8 @@ import datetime
 import io
 import shlex  # Simple lexical analysis
 from os.path import expanduser
+import tempfile as tmpf
+import subprocess
 
 try:  # We are Python 3.3+
     from tkinter import *
@@ -36,53 +38,64 @@ tipwindow = None
 class App:
     """the GUI graphic application"""
 
-    def __init__(self):
+    def __init__(self, use_gui=True):
         """create a tkk graphic interface with a main window tk_win"""
-        self.__version__ = "0.11.1"
-        self._title = "of 2021-08-09b : 'Script me more !'"
+        self.__version__ = "0.12.1"
+        self._title = "of 2021-08-15a : 'Pop-up results to .excel !'"
         self.conn = None  # Baresql database object
         self.database_file = ""
-        self.tk_win = Tk()
-        self.tk_win.title(
-            "A graphic SQLite Client in 1 Python file (" + self.__version__ + ")"
-        )
-        self.tk_win.option_add("*tearOff", FALSE)  # hint of tk documentation
-        self.tk_win.minsize(600, 200)  # minimal size
-
-        self.font_size = 10
-        self.font_wheight = 0
         self.initialdir = "."
-        # With a Menubar and Toolbar
-        self.create_menu()
-        self.create_toolbar()
 
-        # Create style "ButtonNotebook"
-        self.create_style()
-        # Initiate Drag State
-        self.state_drag = False
-        self.state_drag_index = 0
+        # Do we use a GUI ?
+        self.use_gui = use_gui  # gui ok by default
+        try:
+            self.tk_win = Tk()
+        except:
+            self.use_gui = False
 
-        # With a Panedwindow of two frames: 'Database' and 'Queries'
-        p = ttk.Panedwindow(self.tk_win, orient=HORIZONTAL)
-        p.pack(fill=BOTH, expand=1)
+        if self.use_gui:
+            self.tk_win.title(
+                "A graphic SQLite Client in 1 Python file (" + self.__version__ + ")"
+            )
+            self.tk_win.option_add("*tearOff", FALSE)  # hint of tk documentation
+            self.tk_win.minsize(600, 200)  # minimal size
 
-        f_database = ttk.Labelframe(p, text="Databases", width=200, height=100)
-        p.add(f_database)
-        f_queries = ttk.Labelframe(p, text="Queries", width=200, height=100)
-        p.add(f_queries)
+            self.font_size = 10
+            self.font_wheight = 0
+            # With a Menubar and Toolbar
+            self.create_menu()
+            self.create_toolbar()
 
-        # build tree view 't' inside the left 'Database' Frame
-        self.db_tree = ttk.Treeview(
-            f_database, displaycolumns=[], columns=("detail", "action")
-        )
-        self.db_tree.tag_configure("run")
-        self.db_tree.pack(fill=BOTH, expand=1)
+            # Create style "ButtonNotebook"
+            self.create_style()
+            # Initiate Drag State
+            self.state_drag = False
+            self.state_drag_index = 0
 
-        # create a  notebook 'n' inside the right 'Queries' Frame
-        self.n = NotebookForQueries(self.tk_win, f_queries, [])
+            # With a Panedwindow of two frames: 'Database' and 'Queries'
+            p = ttk.Panedwindow(self.tk_win, orient=HORIZONTAL)
+            p.pack(fill=BOTH, expand=1)
 
-        # Bind keyboard shortcuts
-        self.tk_win.bind("<F9>", self.run_tab)
+            f_database = ttk.Labelframe(p, text="Databases", width=200, height=100)
+            p.add(f_database)
+            f_queries = ttk.Labelframe(p, text="Queries", width=200, height=100)
+            p.add(f_queries)
+
+            # build tree view 't' inside the left 'Database' Frame
+            self.db_tree = ttk.Treeview(
+                f_database, displaycolumns=[], columns=("detail", "action")
+            )
+            self.db_tree.tag_configure("run")
+            self.db_tree.pack(fill=BOTH, expand=1)
+
+            # create a  notebook 'n' inside the right 'Queries' Frame
+            self.n = NotebookForQueries(self.tk_win, f_queries, [], self.use_gui)
+
+            # Bind keyboard shortcuts
+            self.tk_win.bind("<F9>", self.run_tab)
+        else:
+            # create a GUI-Less notebook 'n'
+            self.n = NotebookForQueries(None, None, [], self.use_gui)
 
         # define default home directory
         self.home = expanduser("~")
@@ -98,6 +111,7 @@ class App:
         self.output_file = None
         self.init_output = True
         self.output_mode = False
+        self.x_mode = False
 
     def create_menu(self):
         """create the menu of the application"""
@@ -123,6 +137,12 @@ class App:
         self.menu.add_command(label="Close Database", command=self.close_db)
         self.menu.add_separator()
         self.menu.add_command(label="Attach Database", command=self.attach_db)
+        if sys.version_info[:2] >= (3, 7):
+            self.menu.add_separator()
+            self.menu.add_command(label="Backup main Database", command=self.backup_db)
+            self.menu.add_command(
+                label="Restore into main Database", command=self.restore_db
+            )
         self.menu.add_separator()
         self.menu.add_command(label="Quit", command=self.quit_db)
 
@@ -172,6 +192,12 @@ class App:
             ("sqlin_img", self.load_script, "Load a SQL script file"),
             ("sqlsav_img", self.sav_script, "Save a SQL script in a file"),
             ("chgsz_img", self.chg_fontsize, "Modify font size"),
+            (
+                "img_run_temp",
+                self.run_temp,
+                "Run script selection and Display output in temporary files",
+            ),
+            ("img_clean_temp", self.clean_temp, "Remove old temporary files"),
         ]
 
         for img, action, tip in to_show:
@@ -220,6 +246,43 @@ class App:
             self.conn = Baresql(self.database_file)
             self.actualize_db()
 
+    def backup_db(self, filename="", isolation_level=None):
+        """Backup the current database"""
+        if filename == "":
+            filename = filedialog.asksaveasfilename(
+                initialdir=self.initialdir,
+                defaultextension=".db",
+                title="Define a new database name and location",
+                filetypes=[("default", "*.db"), ("other", "*.db*"), ("all", "*.*")],
+            )
+        if filename != "":
+            if os.path.isfile(filename):
+                self.set_initialdir(filename)
+                if messagebox.askyesno(
+                    message="Confirm Destruction of previous Datas ?",
+                    icon="question",
+                    title="Destroying",
+                ):
+                    os.remove(filename)
+                    db_to = sqlite.connect(filename)
+                    self.conn.conn.backup(db_to)
+                    db_to.close()
+                    self.actualize_db()
+
+    def restore_db(self, filename="", isolation_level=None):
+        """Restore an existing database into current one"""
+        if filename == "":
+            filename = filedialog.askopenfilename(
+                initialdir=self.initialdir,
+                defaultextension=".db",
+                filetypes=[("default", "*.db"), ("other", "*.db*"), ("all", "*.*")],
+            )
+        if filename != "":
+            db_from = sqlite.connect(filename)
+            db_from.backup(self.conn.conn)
+            db_from.close
+            self.actualize_db()
+
     def load_script(self):
         """load a script file, ask validation of detected Python code"""
         filename = filedialog.askopenfilename(
@@ -233,17 +296,22 @@ class App:
             with io.open(filename, encoding=guess_encoding(filename)[0]) as f:
                 script = f.read()
                 sqls = self.conn.get_sqlsplit(script, remove_comments=True)
-                dg = [s for s in sqls if s.strip(" \t\n\r")[:5] == "pydef"]
+                dg = [
+                    s
+                    for s in sqls
+                    if s.strip(" \t\n\r")[:5].lower() in ("pydef", ".read", ".shel")
+                    or s.strip(" \t\n\r")[:1].lower() == "."
+                ]
                 if dg:
                     fields = [
                         "",
                         ["In Script File:", filename, "r", 100],
                         "",
-                        ["Python Script", "".join(dg), "r", 80, 20],
+                        ["non pure SQL code", "\n".join(dg), "r", 80, 20],
                     ]
 
                     create_dialog(
-                        ("Ok for this Python Code ?"),
+                        ("Ok for this non pure SQL code ?"),
                         fields,
                         ("Confirm", self.load_script_ok),
                         [text, script],
@@ -321,6 +389,9 @@ class App:
 
     def actualize_db(self):
         """refresh the database view"""
+        if not self.use_gui:
+            return
+
         # bind double-click for easy user interaction
         self.db_tree.tag_bind("run", "<Double-1>", self.t_doubleClicked)
         self.db_tree.tag_bind("run_up", "<Double-1>", self.t_doubleClicked)
@@ -362,6 +433,12 @@ class App:
 
     def run_tab(self, event=None):
         """clear previous results and run current script of a tab"""
+        if not self.use_gui:
+            active_tab_id = len(self.n.nongui_tabs) - 1
+            script = self.n.nongui_tabs[active_tab_id]
+            # print ("silently running,active_tab_id , script)  # testtmp
+            self.create_and_add_results(script, active_tab_id)
+            return
         active_tab_id = self.n.notebook.select()
         if active_tab_id != "":
             # remove previous results
@@ -398,6 +475,41 @@ class App:
             self.create_and_add_results(script, active_tab_id, limit=99, log=f)
             fw.focus_set()  # workaround bug http://bugs.python.org/issue17511
 
+    def run_temp(self):
+        """run selected script commands and display results via tmp files"""
+        # backup existing defaults
+        once_mode_bkp = self.once_mode
+        encode_in_bkp = self.encode_in
+        output_file_bkp = self.output_file
+        init_output_bkp = self.init_output
+        output_mode_bkp = self.output_mode
+        x_mode_bkp = self.x_mode
+
+        active_tab_id = self.n.notebook.select()
+        if active_tab_id != "":
+            self.n.remove_treeviews(active_tab_id)
+
+        if active_tab_id != "":
+            # get current selection (or all)
+            fw = self.n.fw_labels[active_tab_id]
+            fw = self.n.fw_labels[active_tab_id]
+            try:
+                query = fw.get("sel.first", "sel.last")
+            except:
+                query = fw.get(1.0, END)[:-1]
+            query = "\n.output --bom -x \n" + query
+            # print("run temp", query)
+            self.create_and_add_results(query, active_tab_id)
+            fw.focus_set()  # workaround bug http://bugs.python.org/issue17511
+
+        # restore previous defaults
+        self.once_mode = once_mode_bkp
+        self.encode_in = encode_in_bkp
+        self.output_file = output_file_bkp
+        self.init_output = init_output_bkp
+        self.output_mode = output_mode_bkp
+        self.x_mode = x_mode_bkp
+
     def chg_fontsize(self):
         """change the display font size"""
         sizes = [10, 13, 14]
@@ -427,6 +539,37 @@ class App:
                 size=self.font_size, weight=ww[self.font_wheight], family=ff
             )
 
+    def clean_temp(self):
+        """clear temp directory"""
+        ff = tmpf.NamedTemporaryFile(delete=True, suffix="_sqlite_bro.csv").name
+        temp_directory = os.path.dirname(ff)
+        report = [
+            ("", ""),
+        ]
+        for file in os.listdir(temp_directory):
+            if file.endswith("_sqlite_bro.csv"):
+                print("removing ", os.path.join(temp_directory, file))
+                try:
+                    os.remove(os.path.join(temp_directory, file))
+                    report += [
+                        ("removing ", os.path.join(temp_directory, file)),
+                    ]
+                except PermissionError:
+                    report += [
+                        ("PermissionError ", os.path.join(temp_directory, file)),
+                    ]
+
+        active_tab_id = self.n.notebook.select()
+        if active_tab_id != "":
+            self.n.remove_treeviews(active_tab_id)
+        self.n.add_treeview(
+            active_tab_id,
+            ("Cleanup", "file"),
+            (report),
+            "Cleanup",
+            ".Cleaning tmp files",
+        )
+
     def t_doubleClicked(self, event):
         """launch action when dbl_click on the Database structure"""
         # determine item to consider
@@ -523,6 +666,43 @@ GDBEA0qNJyGw7AmxmuaZhWEU5kEJADs=
 """,
             "img_closepressed": """\
 R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
+""",
+            "img_run_temp": """\
+R0lGODlhGAAYAHAAACH5BAEAAPwALAAAAAAYABgAhwAAAAAAMwAAZgAAmQAAzAAA/wArAAArMwAr
+ZgArmQArzAAr/wBVAABVMwBVZgBVmQBVzABV/wCAAACAMwCAZgCAmQCAzACA/wCqAACqMwCqZgCq
+mQCqzACq/wDVAADVMwDVZgDVmQDVzADV/wD/AAD/MwD/ZgD/mQD/zAD//zMAADMAMzMAZjMAmTMA
+zDMA/zMrADMrMzMrZjMrmTMrzDMr/zNVADNVMzNVZjNVmTNVzDNV/zOAADOAMzOAZjOAmTOAzDOA
+/zOqADOqMzOqZjOqmTOqzDOq/zPVADPVMzPVZjPVmTPVzDPV/zP/ADP/MzP/ZjP/mTP/zDP//2YA
+AGYAM2YAZmYAmWYAzGYA/2YrAGYrM2YrZmYrmWYrzGYr/2ZVAGZVM2ZVZmZVmWZVzGZV/2aAAGaA
+M2aAZmaAmWaAzGaA/2aqAGaqM2aqZmaqmWaqzGaq/2bVAGbVM2bVZmbVmWbVzGbV/2b/AGb/M2b/
+Zmb/mWb/zGb//5kAAJkAM5kAZpkAmZkAzJkA/5krAJkrM5krZpkrmZkrzJkr/5lVAJlVM5lVZplV
+mZlVzJlV/5mAAJmAM5mAZpmAmZmAzJmA/5mqAJmqM5mqZpmqmZmqzJmq/5nVAJnVM5nVZpnVmZnV
+zJnV/5n/AJn/M5n/Zpn/mZn/zJn//8wAAMwAM8wAZswAmcwAzMwA/8wrAMwrM8wrZswrmcwrzMwr
+/8xVAMxVM8xVZsxVmcxVzMxV/8yAAMyAM8yAZsyAmcyAzMyA/8yqAMyqM8yqZsyqmcyqzMyq/8zV
+AMzVM8zVZszVmczVzMzV/8z/AMz/M8z/Zsz/mcz/zMz///8AAP8AM/8AZv8Amf8AzP8A//8rAP8r
+M/8rZv8rmf8rzP8r//9VAP9VM/9VZv9Vmf9VzP9V//+AAP+AM/+AZv+Amf+AzP+A//+qAP+qM/+q
+Zv+qmf+qzP+q///VAP/VM//VZv/Vmf/VzP/V////AP//M///Zv//mf//zP///wAAAAAAAAAAAAAA
+AAiFAPcJHEiwoMGDBgEo3Lew4UKEAwEIlEiRoUWIEzNmVCgR48WOFkF6HEmypMmTAqWpXIky5Upp
+LV2qPDikps2bNV/CNFhzH06bLGfyHDIQ50qi+3YW7Ekwp0qmCKESfCp1KESbGKsuzZAVKU2tTb0S
+zHCz69KfZgX+xHqVKNm1YIvCxYkxIAA7
+""",
+            "img_clean_temp": """\
+R0lGODlhGAAYAHAAACwAAAAAGAAYAIcAAAAAADMAAGYAAJkAAMwAAP8AKwAAKzMAK2YAK5kAK8wA
+K/8AVQAAVTMAVWYAVZkAVcwAVf8AgAAAgDMAgGYAgJkAgMwAgP8AqgAAqjMAqmYAqpkAqswAqv8A
+1QAA1TMA1WYA1ZkA1cwA1f8A/wAA/zMA/2YA/5kA/8wA//8zAAAzADMzAGYzAJkzAMwzAP8zKwAz
+KzMzK2YzK5kzK8wzK/8zVQAzVTMzVWYzVZkzVcwzVf8zgAAzgDMzgGYzgJkzgMwzgP8zqgAzqjMz
+qmYzqpkzqswzqv8z1QAz1TMz1WYz1Zkz1cwz1f8z/wAz/zMz/2Yz/5kz/8wz//9mAABmADNmAGZm
+AJlmAMxmAP9mKwBmKzNmK2ZmK5lmK8xmK/9mVQBmVTNmVWZmVZlmVcxmVf9mgABmgDNmgGZmgJlm
+gMxmgP9mqgBmqjNmqmZmqplmqsxmqv9m1QBm1TNm1WZm1Zlm1cxm1f9m/wBm/zNm/2Zm/5lm/8xm
+//+ZAACZADOZAGaZAJmZAMyZAP+ZKwCZKzOZK2aZK5mZK8yZK/+ZVQCZVTOZVWaZVZmZVcyZVf+Z
+gACZgDOZgGaZgJmZgMyZgP+ZqgCZqjOZqmaZqpmZqsyZqv+Z1QCZ1TOZ1WaZ1ZmZ1cyZ1f+Z/wCZ
+/zOZ/2aZ/5mZ/8yZ///MAADMADPMAGbMAJnMAMzMAP/MKwDMKzPMK2bMK5nMK8zMK//MVQDMVTPM
+VWbMVZnMVczMVf/MgADMgDPMgGbMgJnMgMzMgP/MqgDMqjPMqmbMqpnMqszMqv/M1QDM1TPM1WbM
+1ZnM1czM1f/M/wDM/zPM/2bM/5nM/8zM////AAD/ADP/AGb/AJn/AMz/AP//KwD/KzP/K2b/K5n/
+K8z/K///VQD/VTP/VWb/VZn/Vcz/Vf//gAD/gDP/gGb/gJn/gMz/gP//qgD/qjP/qmb/qpn/qsz/
+qv//1QD/1TP/1Wb/1Zn/1cz/1f///wD//zP//2b//5n//8z///8AAAAAAAAAAAAAAAAIcwD3CRxI
+sKDBgwgTKlzIUKEfPw0T/qES8SAViBUL+vmTsSAVjh0H/sEYct/FkgJHojRJMqM9P1VaVrRHE2ZJ
+e/BqhsRJM2fHnD1puuQJ9GdQewIBKN23tOnSfTR5FgSQlKlVqlQXZs169anCrQOxrhyLMCAAOw==
 """,
         }
         return {k: PhotoImage(k, data=v) for k, v in icons.items()}
@@ -829,6 +1009,8 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                 try:
                     if shell_list[0] == ".cd" and len(shell_list) >= 2:
                         db_file = shell_list[1]
+                        db_file = db_file.strip("'")
+                        db_file = db_file.strip('"')
                         if (db_file + "z")[0] == "~":
                             self.current_directory = os.path.join(
                                 self.home, db_file[1:]
@@ -852,11 +1034,14 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                             self.default_header = True
                     if shell_list[0] == ".separator" and len(shell_list) >= 2:
                         self.default_separator = shell_list[1]
-                    if shell_list[0] in (".once", ".output"):
+                    if shell_list[0] in (".once", ".output", ".excel"):
+                        if shell_list[0] == ".excel":
+                            shell_list = [".once", "--bom", "-x"]
                         if shell_list[0] == ".once":
                             self.once_mode, self.init_output = True, True
                         else:
                             self.output_mode, self.init_output = True, True
+                        self.x_mode = False
                         self.encode_in = "utf-8"
                         if "--bom" in shell_list:  # keep access to the option
                             self.encode_in = "utf-8-sig"
@@ -864,6 +1049,15 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                             self.output_file = shell_list[2]
                         else:
                             self.output_file = shell_list[1]
+                        self.output_file = self.output_file.strip("'")
+                        self.output_file = self.output_file.strip('"')
+                        if (self.output_file + "") == "-x":
+                            self.x_mode = True
+                            with tmpf.TemporaryFile() as ff:
+                                ff = tmpf.NamedTemporaryFile(
+                                    delete=False, suffix="_sqlite_bro.csv"
+                                )
+                                self.output_file = ff.name
                         if (self.output_file + "z")[0] == "~":
                             self.output_file = os.path.join(
                                 self.home, self.output_file[1:]
@@ -878,10 +1072,12 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                             with io.open(
                                 self.output_file, write_mode, encoding=self.encode_in
                             ) as fout:
-                                fout.writelines(instru[len(".print") + 1:] + "\n")
+                                fout.writelines(instru[len(".print") + 1 :] + "\n")
                             self.init_output, self.once_mode = False, False
                     if shell_list[0] == ".import" and len(shell_list) >= 2:
                         csv_file = shell_list[1]
+                        csv_file = csv_file.strip("'")
+                        csv_file = csv_file.strip('"')
                         if (csv_file + "z")[0] == "~":
                             csv_file = os.path.join(self.home, csv_file[1:])
                         guess = guess_csv(csv_file)
@@ -926,6 +1122,8 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                     if shell_list[0] == ".dump":
                         if len(shell_list) >= 2:
                             csv_file = shell_list[1]
+                            csv_file = csv_file.strip("'")
+                            csv_file = csv_file.strip('"')
                             if (csv_file + "z")[0] == "~":
                                 csv_file = os.path.join(self.home, csv_file[1:])
                             with io.open(csv_file, "w", encoding="utf-8") as f:
@@ -941,6 +1139,8 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                             )
                     if shell_list[0] == ".read" and len(shell_list) >= 2:
                         filename = shell_list[1]
+                        filename = filename.strip("'")
+                        filename = filename.strip('"')
                         if (filename + "z")[0] == "~":
                             filename = os.path.join(self.home, filename[1:])
                         with io.open(
@@ -965,8 +1165,7 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                         if (filename + "z")[0] == "~":
                             filename = os.path.join(self.home, filename[1:])
                         db_from = sqlite.connect(filename)
-                        with db_from:
-                            db_from.backup(self.conn.conn)
+                        db_from.backup(self.conn.conn)
                         db_from.close
                         self.actualize_db()
                     if shell_list[0] == ".backup" and len(shell_list) >= 2:
@@ -974,23 +1173,27 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                         if (filename + "z")[0] == "~":
                             filename = os.path.join(self.home, filename[1:])
                         db_to = sqlite.connect(filename)
-                        with db_to:
-                            self.conn.conn.backup(db_to)
+                        self.conn.conn.backup(db_to)
                         db_to.close()
+                    if shell_list[0] == ".shell" and len(shell_list) >= 2:
+                        os.system(instru[len(".print") + 1 :] + "\n")
 
                 except IOError as err:
                     msg = "I/O error: {0}".format(err)
                     self.n.add_treeview(
                         tab_tk_id, ("Error !",), [(msg,)], "Error !", instru
                     )
+                    if not self.use_gui:
+                        print("Error !", [msg])
                     if log is not None:  # write to logFile
                         log.write("Error ! %s : %s" % (msg, instru))
                     sql_error = True
                     break
             elif len("".join(instruction.split())) > 1:  # PyPy answer 42 to blanks sql
+                nb_columns = -1
                 try:
                     if self.output_mode or self.once_mode:
-                        self.conn.export_writer(
+                        nb_columns = self.conn.export_writer(
                             instruction,
                             self.output_file,
                             header=self.default_header,
@@ -998,14 +1201,26 @@ R0lGODdhCAAIAIgAAPAAAP///ywAAAAACAAIAAACDkyAeJYM7FR8Ex7aVpIFADs=
                             encoding=self.encode_in,
                             initialize=self.init_output,
                         )
-                        self.once_mode, self.init_output = False, False
-                        self.n.add_treeview(
-                            tab_tk_id,
-                            ("qry", "file"),
-                            ((instruction, self.output_file),),
-                            "Info",
-                            ".once %s" % self.output_file,
-                        )
+                        if nb_columns > 0:
+                            self.once_mode, self.init_output = False, False
+                        if nb_columns > 0:
+                            self.n.add_treeview(
+                                tab_tk_id,
+                                ("qry_to_csv", "file"),
+                                ((instruction, self.output_file),),
+                                "Qry",
+                                # ".once %s" % self.output_file,
+                                first_line,
+                            )
+                        if self.x_mode and nb_columns > 0:
+                            os.system(
+                                "start excel.exe " + self.output_file.replace("\\", "/")
+                            )
+                            with tmpf.TemporaryFile() as ff:
+                                ff = tmpf.NamedTemporaryFile(
+                                    delete=False, suffix="_sqlite_bro.csv"
+                                )
+                                self.output_file = ff.name
                     else:
                         cur = cu.execute(instruction)
                         rows = cur.fetchall()
@@ -1172,23 +1387,32 @@ class NotebookForQueries:
     """Create a Notebook with a list in the First frame
     and query results in following treeview frames"""
 
-    def __init__(self, tk_win, root, queries):
-        self.tk_win = tk_win
-        self.root = root
-        self.notebook = Notebook(root, style="ButtonNotebook")  # ttk.
+    def __init__(self, tk_win, root, queries, use_gui):
+        self.use_gui = use_gui
+        self.nongui_tabs = [
+            "",
+        ]
+        if self.use_gui:
+            self.tk_win = tk_win
+            self.root = root
+            self.notebook = Notebook(root, style="ButtonNotebook")  # ttk.
 
-        self.fw_labels = {}  # tab_tk_id -> Scripting frame python object
-        self.fw_result_nbs = {}  # tab_tk_id -> Notebook of Results
+            self.fw_labels = {}  # tab_tk_id -> Scripting frame python object
+            self.fw_result_nbs = {}  # tab_tk_id -> Notebook of Results
 
-        # resize rules
-        root.columnconfigure(0, weight=1)
-        root.rowconfigure(0, weight=1)
-        # grid widgets
-        self.notebook.grid(row=0, column=0, sticky=(N, W, S, E))
+            # resize rules
+            root.columnconfigure(0, weight=1)
+            root.rowconfigure(0, weight=1)
+            # grid widgets
+            self.notebook.grid(row=0, column=0, sticky=(N, W, S, E))
 
     def new_query_tab(self, title, query):
         """add a Tab 'title' to the notebook, containing the Script 'query'"""
 
+        if not self.use_gui:
+            self.nongui_tabs += [query]
+            return len(self.nongui_tabs) - 1
+
         fw_welcome = ttk.Panedwindow(self.tk_win, orient=VERTICAL)  # tk_win
         fw_welcome.pack(fill="both", expand=True)
         self.notebook.add(fw_welcome, text=(title))
@@ -1244,6 +1468,8 @@ class NotebookForQueries:
 
     def add_treeview(self, given_tk_id, columns, data, title="__", subt=""):
         """add a dataset result to the given tab tk_id"""
+        if not self.use_gui:
+            return
         # ensure we work on lists
         if isinstance(columns, (type("e"), type("e"))):
             tree_columns = [columns]
@@ -1896,9 +2122,14 @@ class Baresql:
         quotechar='"',
         initialize=True,
     ):
-        """export a csv table (action)"""
+        """export a csv table and return number of columns"""
         cursor = self.conn.cursor()
         cursor.execute(sql)
+        # do nothing if nothing
+        if cursor.description is None or len(cursor.description) == 0:
+            return -1
+        else:
+            nb_columns = len(cursor.description)
         # with PyPy, the "with io.open" for is more than necessary
         if sys.version_info[0] != 2:  # python3
             write_mode = "w" if initialize else "a"  # Write or Append
@@ -1911,6 +2142,7 @@ class Baresql:
                         [i if isinstance(i, str) else i[0] for i in cursor.description]
                     )  # PyPy as a strange list of list
                 writer.writerows(cursor.fetchall())
+                fout.close  # PyPy3-7.3.5 needs that close
         else:  # python2.7 (minimal)
             write_mode = "wb" if initialize else "ab"  # Write or Append
             with io.open(csv_file, write_mode) as fout:
@@ -1925,6 +2157,8 @@ class Baresql:
                         [i if isinstance(i, str) else i[0] for i in cursor.description]
                     )  # heading row with anti-PyPy bug
                 writer.writerows(cursor.fetchall())
+                fout.close  # PyPy3-7.3.5 needs that close
+        return nb_columns
 
 
 def _main():
@@ -1943,25 +2177,17 @@ CREATE VIEW v1 as select * from item inner join part as p ON ItemNo=p.ParentNo;
 INSERT INTO item values("T","Ford",1000);
 INSERT INTO item select "A","Merced",1250 union all select "W","Wheel",9 ;
 INSERT INTO part select ItemNo,"W","needed",Kg/250 from item where Kg>250;
-\n-- to CREATE a Python embedded function (enclose them by "py" and ";") :
+\n-- to CREATE a Python embedded function, enclose them by "py" and ";" :
 pydef py_hello():
     "hello world"
     return ("Hello, World !");
-pydef py_sin(s):
-    "sinus function : example loading module, handling input/output as strings"
-    import math as py_math
-    return ("%s" %  py_math.sin(s*1));
 pydef py_fib(n):
    "fibonacci : example with function call (may only be internal) "
    fib = lambda n: n if n < 2 else fib(n-1) + fib(n-2)
    return("%s" % fib(n*1));
-pydef py_power(x,
-               y):
-    "power function : example loading module, handling input/output as strings"
-    import math as py_math
-    return ("%s" %  ((x*1) ** (y*1)) );
-\n-- to USE a python embedded function and nesting of embedded functions:
-select py_hello(), py_sin(1) as sinus, py_power(2, 1*py_fib(6)) as power, sqlite_version();
+
+-- to USE a python embedded function and nesting of embedded functions:
+select py_hello(), py_fib(6) as fibonacci, sqlite_version();
 \n-- to EXPORT :
 --    a TABLE, select TABLE, then click on icon 'SQL->CSV'
 --    a QUERY RESULT, select the SCRIPT text, then click on icon '???->CSV',
@@ -1984,9 +2210,10 @@ SELECT ItemNo, Description FROM Item;  -- see all is back to normal
 RELEASE SAVEPOINT remember_Neo; -- free memory
 
 \n\n-- '.' commands understood:
--- .backup FILE           Backup DB (default "main") to FILE
+-- .backup FILE           Backup DB (default "main") to FILE (if Python>=3.7)
 -- .cd DIRECTORY          Change the working directory to DIRECTORY
 -- .dump ?FILE?           Render database content as SQL (to FILE if specified)
+-- .excel                 Display the output of next command in spreadsheet
 -- .headers on|off        Turn display of headers on or off
 -- .import FILE TABLE     Import data from FILE into TABLE
 --                        (create TABLE only if it doesn't exist, keep existing records)
@@ -1995,26 +2222,23 @@ RELEASE SAVEPOINT remember_Neo; -- free memory
 -- .output ?FILE?         Send output to FILE or stdout if FILE is omitted
 -- .print STRING...       Print literal STRING
 -- .read FILE             Read input from FILE
--- .restore FILE          Restore content of DB (default "main") from FILE
+-- .restore FILE          Restore DB (default "main") from FILE (if Python>=3.7)
 -- .separator COL         Set column separator in next .once exports (default ,)
+-- .shell CMD ARGS...     Run CMD ARGS... in a system shell
 
 .headers on
 .separator ;
-.once --bom '~this_file_of_result.txt'
+.once --bom  '~this_file_of result.txt'
 select ItemNo, Description from item order by ItemNo desc;
-.import '~this_file_of_result.txt' in_this_table
+.import '~this_file_of result.txt' in_this_table
 .cd ~
 ATTACH 'test.db' as toto;
 DROP TABLE IF EXISTS toto.new_item;
 CREATE TABLE toto.new_item as select * from "main"."item";
 .dump
 """
-    # print(args)
-    app = App()
-    # start with a memory Database and a welcome
-    app.new_db(":memory:")
 
-    if "argparse" in globals():
+    if "argparse" in globals():  # not before Python-3.2
         parser = argparse.ArgumentParser(
             description="sqlite_bro : a graphic SQLite browser in 1 Python file"
         )
@@ -2038,6 +2262,13 @@ CREATE TABLE toto.new_item as select * from "main"."item";
             "-sc", "--scripts", type=str, help="qive a list of initial scripts"
         )
         args = parser.parse_args()
+
+        if args.quiet:
+            app = App(use_gui=False)
+        else:
+            app = App(use_gui=True)
+        # start with a memory Database and a welcome
+        app.new_db(":memory:")
         if args.database:
             app.open_db(args.database)
         if args.scripts:
@@ -2057,8 +2288,12 @@ CREATE TABLE toto.new_item as select * from "main"."item";
         if args.quiet:
             app.close_db
     else:
+        app = App(use_gui=True)
+        # start with a memory Database and a welcome
+        app.new_db(":memory:")
         app.n.new_query_tab("Welcome", welcome_text)
-    app.tk_win.mainloop()
+    if app.use_gui:
+        app.tk_win.mainloop()
 
 
 if __name__ == "__main__":
